package RPM::Specfile::Multispec;

use strict;
use RPM::Specfile;

use vars qw/$VERSION @ISA/;

$VERSION = '1.03';
@ISA = qw/RPM::Specfile/;

sub push_package {
  my $self = shift;
  $self->{__package__} ||= [ ];
  foreach my $package (@_) {
      $package->_is_package(1);
      push @{ $self->{__package__} }, $package;
  }
}

sub clear_package {
  my $self = shift;
  $self->{__package__} = [ ];
}

sub package {
  my $self = shift;
  $self->{__package__} ||= [ ];
  if (@_) {
    my $index = shift;
    if (@_) {
      my $value = shift;
      $self->{__package__}->[$index] = $value;
    }
    return $self->{__package__}->[$index];
  }
  else {
    return @{$self->{__package__}};
  }
}

sub generate_specfile {
  my $self = shift;

  my $output;

  $output .= $self->_generate_specfile_header();
  map {$output .= $_->_generate_specfile_header() } $self->package();

  $output .= $self->_generate_specfile_common();

  $output .= $self->_generate_specfile_triggers();
  map {$output .= $_->_generate_specfile_triggers() } $self->package();

  $output .= $self->_generate_specfile_files();
  map {$output .= $_->_generate_specfile_files() } $self->package();

  $output .= $self->_generate_specfile_changelog();

  return $output;
}






### Private methods ###
# WARNING!
# They are subject to change without notice 
#######################

sub _is_package {
  my $self = shift;
  my $_is_package = shift;
  if (defined $_is_package) {
    $self->{___IS_PACKAGE__} = $_is_package ? 1 : 0;
  }
  return $self->{___IS_PACKAGE__} ? 1 : 0;
}


sub _generate_specfile_header {
  my $self = shift;
  
  my $output;

  my %defaults = ();
#    ( buildroot => "%{_tmppath}/%{name}-%{version}-%{release}-root" );
  $self->$_($self->$_() || $defaults{$_}) foreach keys %defaults;

  my %proper_names = ( url           => 'URL',
                       buildroot     => 'BuildRoot',
                       buildrequires => 'BuildRequires',
                       buildarch     => 'BuildArch',
                     );

  #
  # Add any macro definitions to the begining.
  $output .= $self->macros() . "\n" if defined $self->macros();

  my @header_tags=qw/summary name version release epoch packager vendor distribution license group url buildroot buildarch requires conflicts buildrequires/;

  if ($self->_is_package()) {
    @header_tags=qw/summary version release epoch license group url requires conflicts buildrequires/;
    $output .= '%package -n ' . $self->name() . "\n";
  }

  foreach my $tag (@header_tags) {
    my $proper = $proper_names{$tag} || ucfirst $tag;

    next unless defined $self->$tag();
    $output .= "$proper: " . $self->$tag() . "\n";
  }

  my @reqs = $self->buildrequire;
  for my $i (0 .. $#reqs) {
    $output .= "BuildRequires: $reqs[$i]\n";
  }

  @reqs = $self->require;
  for my $i (0 .. $#reqs) {
    $output .= "Requires: $reqs[$i]\n";
  }

  my @sources = $self->source;
  for my $i (0 .. $#sources) {
    $output .= "Source$i: $sources[$i]\n";
  }

  my @patches = $self->patch;
  for my $i (0 .. $#patches) {
    $output .= "Patch$i: $patches[$i]\n";
  }

  #
  # Add any prefixes:
  my @prefixes = $self->prefix;
  for my $i (0 .. $#prefixes) {
    $output .= "Prefix: $prefixes[$i]\n";
  }
  $output .= "\n";

  $output .= $self->_generate_specfile_section(qw/description/);

  return $output;
}

sub _generate_specfile_common {
  my $self = shift;

  my $output;

  #
  # Add patch entries to the %prep section if they exist:
  my $prep = $self->prep();
  my @patches = $self->patch;
  for my $i (0 .. $#patches) {
    $prep .= "\n" if($i == 0);		# Just in case they did not add a newline
    $prep .= "%patch${i} -p1\n"; 
  }
  $self->prep($prep) if(defined($prep));

  foreach my $sect (qw/prep build install clean/) {
    $output .= $self->_generate_specfile_section($sect);
  }
  return $output;
}

sub _generate_specfile_triggers {
  my $self = shift;

  my $output;

  foreach my $sect (qw/pre post preun postun/) {
    $output .= $self->_generate_specfile_section($sect);
  }
  return $output;
}

sub _generate_specfile_section {
  my $self = shift;
  my $sect = shift;
 
  my $output;

  return '' if(!defined($self->$sect())); 
  $output .= "%$sect".($self->_is_package() ? ' -n ' . $self->name() : '')."\n";
  my $content = $self->$sect();
  # remove leading and trailing whitespace and spurious linefeeds
  $content =~ s/^\s*\n*//s;
  $content =~ s/[\s\n]*$/\n\n/s;
  $output .= $content;
  return $output;
}

sub _generate_specfile_files {
  my $self = shift;
  my $output = "\n";
  $output .= "%files ";
  $output .= '-n ' . $self->name if ($self->_is_package);
  $output .= $self->file_param . "\n" if ($self->file_param);
  $output .= "\n";
  $output .= "$_\n" foreach $self->file;
  return $output;
}

sub _generate_specfile_changelog {
  my $self = shift;
  my $output;
  $output .= "\n%changelog\n";
  $output .= "$_\n" foreach $self->changelog;

  return $output;
}

1;

__END__
# Below is stub documentation for your module. You better edit it!
# TODO: yes, I better edit this better.

=head1 NAME

RPM::Specfile::Multispec - Perl extension for creating RPM Specfiles

=head1 SYNOPSIS

  use RPM::Specfile::Multispec;

=head1 DESCRIPTION

RPM::Specfile is a simple module for creation of RPM Spec files.  
RPM::Specfile::Multispec extends its functionality, adding 
List Accessors Methods for manipulating sub-packages in spec file,
and provides improved generate_specfile() method with sub-packages
support.

See L<RPM::Specfile> for details of interface to those methods.
Added List Accessors Methods for manipulating sub-packages are:
package, package_push, package_clean.

=head1 NOTE

A good example of the use of RPM::Specfile::Multispec is debian2spec 
which comes with the RPM::Specfile::Multispec cpan archive.

=head1 AUTHOR

Igor Vlasenko <viy@altlinux.org>

=head1 SEE ALSO

L<RPM::Specfile>
Maximum RPM (http://www.rpm.org/max-rpm/)

=cut
